#ifndef EP_LIB_WIN_HPP_INCLUDED
#define EP_LIB_WIN_HPP_INCLUDED

namespace ep_lib
{

  typedef void* MPI_Datatype;
  typedef void* MPI_Op;


  int MPI_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);
  int MPI_Win_free(MPI_Win *win);
  int MPI_Win_fence(int assert, MPI_Win win);

  int MPI_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
  	          int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

  int MPI_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
              int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

  int MPI_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
  	          int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

  int MPI_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
              int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

  int MPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
  	                 int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

  int MPI_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
                     int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

  int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, 
                         int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp,
                         int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
  
  int MPI_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, 
                         int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp,
                         int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

  int MPI_Win_allocate (MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

  int MPI_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, 
                      MPI_Aint target_disp, MPI_Op op, MPI_Win win);

  int MPI_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, 
                           int target_rank, MPI_Aint target_disp, MPI_Win win);

}

#endif // EP_LIB_COLLECTIVE_HPP_INCLUDED
